#include "playerinterface.hpp"
#include <SFML/System/Vector2.hpp>
#include <SFML/Window/Keyboard.hpp>
#include <iostream>

PlayerDirection GetPlayerInput(void)
{
    if (sf::Keyboard::isKeyPressed(sf::Keyboard::Left) 
            || sf::Keyboard::isKeyPressed(sf::Keyboard::A))
        return kLeft;
    if (sf::Keyboard::isKeyPressed(sf::Keyboard::Up)
            || sf::Keyboard::isKeyPressed(sf::Keyboard::W))
        return kUp;
    if (sf::Keyboard::isKeyPressed(sf::Keyboard::Down)
            || sf::Keyboard::isKeyPressed(sf::Keyboard::S))
        return kDown;
    if (sf::Keyboard::isKeyPressed(sf::Keyboard::Right)
            || sf::Keyboard::isKeyPressed(sf::Keyboard::D))
        return kRight;
    return kNone;
}

bool PlayerOutput::IsOpen(void)
{
    return isWindowAlive;
}

PlayerOutput::PlayerOutput(void)
{
    float kWidth = 1025;
    float kHeight = 725;
    float kBoardWidth = kWidth / kGridSize;
    float kBoardHeight = kHeight / kGridSize;
    gameBoundaries = sf::Vector2f(kBoardWidth, kBoardHeight); 
    gameVideoSettings = sf::VideoMode(kWidth, kHeight);
    drawObject.setSize(sf::Vector2f(kGridSize, kGridSize));
    return;
}

void PlayerOutput::CheckContinue()
{
    DisplayEndScreen();
    while (true)
    {
        gameWindow.pollEvent(event);
        if ((event.type == sf::Event::Closed)
                || (sf::Keyboard::isKeyPressed(sf::Keyboard::Escape)))
        {
            gameWindow.close();
            isWindowAlive = false;
            return;
        }
        if (sf::Keyboard::isKeyPressed(sf::Keyboard::Enter)) { return; }
        sf::sleep(delay);
    }
}

void PlayerOutput::DisplayEndScreen(void)
{
    gameWindow.clear();
    sf::Vector2f textPosition(gameBoundaries);
    textPosition.x = textPosition.x / 2;
    textPosition.y = textPosition.y / 2;
    sf::Font font;
    font.loadFromFile("Arial.ttf");
    sf::Text gameOverText("Game Over\nPress 'Enter' to play again", font);
    gameOverText.setPosition(textPosition);
    gameWindow.draw(gameOverText);
    gameWindow.display();
    return;
}

void PlayerOutput::DisplayScore(int score) {
    sf::Vector2f textPosition(gameBoundaries);
    textPosition.x = textPosition.x / 2;
    textPosition.y = textPosition.y / 2;
    sf::Font font;
    font.loadFromFile("Arial.ttf");
    std::string text = "Score: " + std::to_string(score);
    sf::Text ScoreText(text, font);
    ScoreText.setPosition(textPosition);
    gameWindow.draw(ScoreText);
    
}

void PlayerOutput::DisplayGameState(std::vector< std::vector<GameSpace> >& gameBoard, int score)
{
    CheckWindowEvents();
    if (delay == sf::milliseconds(0)) { return; }
    char* letterOnBoard;
    for (float y = 0; y < gameBoundaries.y; y++)
    {
        for (float x = 0; x < gameBoundaries.x; x++)
        {
            if (gameBoard.at(y).at(x).m_bSnake)
                    DrawSnake(sf::Vector2f(x, y));
            else if (gameBoard.at(y).at(x).m_bFood)
                    DrawFood(sf::Vector2f(x,y));
            else
                    DrawEmpty(sf::Vector2f(x,y));
        }
    }
    DisplayScore(score);
    gameWindow.display();
    sf::sleep(delay);
    return;
}

void PlayerOutput::StartGameWindow(void)
{
    gameWindow.create(gameVideoSettings, "SnakePlusPlus");
    isWindowAlive = true;
    return;
}

void PlayerOutput::SetShowGame(bool isShowing) {
    if (isShowing) { delay = sf::milliseconds(5); }
    else { delay = sf::milliseconds(0); }
    return;
}

void PlayerOutput::CheckWindowEvents(void)
{
    while (gameWindow.pollEvent(event))
    {
        if ((event.type == sf::Event::Closed)
                || (sf::Keyboard::isKeyPressed(sf::Keyboard::Escape))) {
            gameWindow.close();
            isWindowAlive = false;
        }
        if (sf::Keyboard::isKeyPressed(sf::Keyboard::Equal)) { 
            if (delay > sf::milliseconds(16)) { continue; }
            delay += sf::milliseconds(1);
        }
        if (sf::Keyboard::isKeyPressed(sf::Keyboard::Hyphen)) { 
            if (delay == sf::milliseconds(0)) { continue; }
            delay -= sf::milliseconds(1);
        }
    }
}

void PlayerOutput::DrawEmpty(sf::Vector2f location)
{
    location *= static_cast<float>(kGridSize);
    drawObject.setPosition(location);
    drawObject.setFillColor(sf::Color::Black);
    gameWindow.draw(drawObject);
    return;
}

void PlayerOutput::DrawFood(sf::Vector2f location)
{
    location *= static_cast<float>(kGridSize);
    drawObject.setPosition(location);
    drawObject.setFillColor(sf::Color::Red);
    gameWindow.draw(drawObject);
    return;
}

void PlayerOutput::DrawSnake(sf::Vector2f location)
{
    location *= static_cast<float>(kGridSize);
    drawObject.setPosition(location);
    drawObject.setFillColor(sf::Color::Green);
    gameWindow.draw(drawObject);
    return;
}
